import { createMap, forMember, mapFrom, Mapper } from '@automapper/core';
import { AutomapperProfile, InjectMapper } from '@automapper/nestjs';
import { Injectable } from '@nestjs/common';
import { Ad } from '../domain/entities/ad';
import { AdPresenter } from '../adapters/primaries/ad.presenter';
import { CreateAdRequest } from '../domain/dtos/create-ad.request';
import { AdCreation } from '../domain/dtos/ad.creation';
import { FrequencyNormaliser } from '../domain/entities/frequency.normaliser';
import { Day } from '../domain/types/day.enum';

@Injectable()
export class AdProfile extends AutomapperProfile {
  frequencyNormaliser = new FrequencyNormaliser();
  constructor(@InjectMapper() mapper: Mapper) {
    super(mapper);
  }
  override get profile() {
    return (mapper) => {
      createMap(mapper, Ad, AdPresenter);
      createMap(
        mapper,
        CreateAdRequest,
        AdCreation,

        forMember(
          (destination) => destination.monMargin,
          mapFrom((source) => source.marginDurations.mon),
        ),
        forMember(
          (destination) => destination.tueMargin,
          mapFrom((source) => source.marginDurations.tue),
        ),
        forMember(
          (destination) => destination.wedMargin,
          mapFrom((source) => source.marginDurations.wed),
        ),
        forMember(
          (destination) => destination.thuMargin,
          mapFrom((source) => source.marginDurations.thu),
        ),
        forMember(
          (destination) => destination.friMargin,
          mapFrom((source) => source.marginDurations.fri),
        ),
        forMember(
          (destination) => destination.satMargin,
          mapFrom((source) => source.marginDurations.sat),
        ),
        forMember(
          (destination) => destination.sunMargin,
          mapFrom((source) => source.marginDurations.sun),
        ),
        forMember(
          (destination) => destination.monTime,
          mapFrom((source) => source.schedule.mon),
        ),
        forMember(
          (destination) => destination.tueTime,
          mapFrom((source) => source.schedule.tue),
        ),
        forMember(
          (destination) => destination.wedTime,
          mapFrom((source) => source.schedule.wed),
        ),
        forMember(
          (destination) => destination.thuTime,
          mapFrom((source) => source.schedule.thu),
        ),
        forMember(
          (destination) => destination.friTime,
          mapFrom((source) => source.schedule.fri),
        ),
        forMember(
          (destination) => destination.satTime,
          mapFrom((source) => source.schedule.sat),
        ),
        forMember(
          (destination) => destination.sunTime,
          mapFrom((source) => source.schedule.sun),
        ),
        forMember(
          (destination) => destination.addresses.create,
          mapFrom((source) => source.addresses),
        ),
        forMember(
          (destination) => destination.fromDate,
          mapFrom((source) =>
            this.frequencyNormaliser.fromDateResolver(source),
          ),
        ),
        forMember(
          (destination) => destination.toDate,
          mapFrom((source) => this.frequencyNormaliser.toDateResolver(source)),
        ),
        forMember(
          (destination) => destination.monTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.mon),
          ),
        ),
        forMember(
          (destination) => destination.tueTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.tue),
          ),
        ),
        forMember(
          (destination) => destination.wedTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.wed),
          ),
        ),
        forMember(
          (destination) => destination.thuTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.thu),
          ),
        ),
        forMember(
          (destination) => destination.friTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.fri),
          ),
        ),
        forMember(
          (destination) => destination.satTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.sat),
          ),
        ),
        forMember(
          (destination) => destination.sunTime,
          mapFrom((source) =>
            this.frequencyNormaliser.scheduleResolver(source, Day.sun),
          ),
        ),
      );
    };
  }
}
